#ifndef _IMAGE__H_
#define _IMAGE__H_
/** @name Classe Image de base
    @memo Classe generique pour le stockage d'image.
    @doc {\bf Fichier a inclure :}
    \begin{verbatim}
    #include "imagetools.H"
    \end{verbatim}
    \\
    La classe Image possde toutes les mthodes standard d'accs, allocation, initialisation etc. De plus, afin d'assurer une compatibilit avec les
    programmes en "C", on peut initialiser avec un buffer dj allou grce  la mthode ShallowSetBuff. Ceci permet d'encapsuler des images de type "C"
    dans des images de type "C++" et ainsi de profiter des algorithmes dj crits.\\

    {\bf Exemple :}
    \begin{verbatim}
    #include "image.H"
    #include "imageio.H"
    #include "imagetools.H"

    int main (void)
    {
    // lecture image PGM
    Image<octet> im;
    ImageIO<octet>::ReadPGM("toto.pgm",im);

    // acces aux pixels
    int i,j;
    for(i=0;i<im.NbRow();i++)
     for(j=0;j<im.NbCol();j++)
       im(i,j)=5;

    // passage a une fonction de type "C"
    algo(im.PixelPointer(0),im.NbRow(),im.NbCol());

    // lecture PPM
    Image<RGB> im2;
    ImageIO<RGB>::ReadPPM("toto.ppm",im2);

    // transfo en YUV
    float *y,*u,*v;
    int h,w;
    ImageTools<RGB>::RGBtoYUV(im2,y,u,v,h,w);

    // transfo du Y en image d octet
    Image<float> imf(h,w,y);
    Image<octet> imo(ImageTools<float>::convertTo8bits(imf));

    // calcul du max
    cout<<"Max="<<(int)ImageTools<octet>::getMax(imo)<<endl;

    // calcul du PSNR entre l original et la composante Y
    cout<<"PSNR="<<ImageTools<octet>::getPSNR(im,imo)<<endl;

    delete []y;
    delete []u;
    delete []v;
    }

    \end{verbatim}
    @author \URL[Thomas Guionnet]{mailto:Thomas.Guionnet@irisa.fr}
    @author \URL[Stephane Pateux]{mailto:Stephane.Pateux@irisa.fr}
    @author \URL[Franck Galpin]{mailto:Franck.Galpin@irisa.fr}
    @see ImageIO ImageTools
*/
//@{

/** Classe Image */
template <class T>
class Image
{
public:

  /** @doc type de la classe Image */
  typedef T Type;

  /** @doc constructeur par defaut */
  Image ();

  /** @doc constructeur
      @param nbRow nombre de lignes
      @param nbCol nombre de colonnes  */
  Image (const int nbRow, const int nbCol);

  /** @doc constructeur
      @param nbRow nombre de lignes
      @param nbCol nombre de colonnes
      @param val valeur de remplissage  */
  Image (const int nbRow, const int nbCol, const T &val);

  /** @doc constructeur par recopie
      @param ima image a recopier */
  Image (const Image &ima);

  /** @doc constructeur par recopie
      @param nbRow nombre de lignes
      @param nbCol nombre de colonnes
      @param buf buffer a recopier  */
  Image (const int nbRow, const int nbCol,  const T *const buf);

   /** @doc destructeur */
  ~Image ();

  /** @name Methodes d'acces aux parametres */
  //@{
  /** @doc taille de l'image
      @return nombre de lignes de l'image */
  int      NbRow () const ;

  /** @doc taille de l'image
      @return nombre de colonnes de l'image */
  int      NbCol () const ;

  /** @doc taille de l'image
      @return taille totale de l'image */
  int      BufferSize () const ;
  //@}

  /**@name Operateurs d'acces en lecture aux donnees */
  //@{
  /** @doc acces aleatoire
      @param row numero de la ligne (de 0 a NbRow-1)
      @param col numero de la colonne (de 0 a NbCol-1)
      @return reference constante sur l'element (row,col) */
  const T& operator () (const int row, const int col) const ;

  /** @doc acces lineaire
      @param i numero du pixel (de 0 a BufferSize-1)
      @return reference constante sur le i eme element */
  const T& operator [] (const int i) const;
  //@}

  /**@name Operateurs d'acces en ecriture aux donnees */
  //@{
  /** @doc acces aleatoire
      @param row numero de la ligne (de 0 a NbRow-1)
      @param col numero de la colonne (de 0 a NbCol-1)
      @return reference sur l'element (row,col) */
  T&       operator () (const int row, const int col);

  /** @doc acces lineaire
      @param i numero du pixel (de 0 a BufferSize-1)
      @return reference constante sur le i eme element */
  T&       operator [] (const int i);

  /** @doc acces aleatoire direct aux donnees
      @param row numero de la ligne (de 0 a NbRow-1)
      @param col numero de la colonne (de 0 a NbCol-1)
      @return pointeur sur l'element (row,col) */
  T*       PixelPointer (const int row, const int col);

  /** @doc acces direct aux donnees
      @param i numero du pixel (de 0 a BufferSize-1)
      @return pointeur sur le i eme element */
  T*       PixelPointer (const int i);

  /** @doc acces aleatoire direct aux donnees en lecture
      @param row numero de la ligne (de 0 a NbRow-1)
      @param col numero de la colonne (de 0 a NbCol-1)
      @return pointeur sur l'element (row,col) */
  const T*       PixelPointer (const int row, const int col) const;

  /** @doc acces direct aux donnees en lecture
      @param i numero du pixel (de 0 a BufferSize-1)
      @return pointeur sur le i eme element */
  const T*       PixelPointer (const int i) const;

  //@}

  /**@name Methodes de modifications des donnees */
  //@{
  /** @doc operateur de recopie
      @param ima image a recopier
      @return reference sur l'image */
  Image&   operator = (const Image &ima);

  /** @doc methode d'initialisation de l'image par remplissage
      @param val valeur de remplissage */
  void     InitTo (const T &val);

  /** @doc methode de retaillage
      @param nbRow nouveau nombre de lignes
      @param nbCol nouveau nombre de colonnes */
  void     Resize (const int nbRow, const int nbCol);

  /** @doc methode de d'initialisation de l'image par recopie
      @param otherBuff buffer a recopier
      @param nbRow nombre de lignes du buffer
      @param nbCol nombre de colonnesdu buffer */
  void     InitBuff(const T*const otherBuff, const int nbRow, const int nbCol);

  /** @doc methode de d'initialisation de l'image par recopie superficielle (ATTENTION A LA GESTION MEMOIRE)
      @param otherBuff buffer a recopier
      @param nbRow nombre de lignes du buffer
      @param nbCol nombre de colonnesdu buffer */
  void     ShallowSetBuff( T* const otherBuff, const int nbRow, const int nbCol);
  //@}

protected:
  /**@name Donnees */
  //@{
  /** @memo contient l'image */
  T*  buffer;

   /** @memo determine si le buffer a ete alloue */
  bool deleteBuffer;

  /** @memo hauteur de l'image */
  int height;

  /** @memo largeur de l'image */
  int width;

  /** @memo taille de l'image */
  int bufferSize;
  //@}

  /**@name Methodes privees */
  //@{
  /** @doc Allocateur d'image
      @param nbRow nombre de lignes du buffer
      @param nbCol nombre de colonnesdu buffer */
  void Allocate(const int nbRow, const int nbCol);

  /** @doc Desallocateur d'image */
  void DeAllocate();
  //@}

}; // end class Image
//@}

#ifndef IMAGE_MAIN
#include "image.cpp"
#endif

#endif // _IMAGE__HH_


