#ifndef _IMAGESTOOLS_MORPHO_H_
#define _IMAGESTOOLS_MORPHO_H_
#include <vector>
#include <algorithm>
#include <list>
#include <iterator>
#include <iostream>
#include <math.h>
using namespace std;

/** @name Outils Morpho Math
    @memo Classe d'outils de morphologie mathematique pour la classe image
    @doc
      Niveau de validite : bon\\
      Niveau de performance :  tres bon\\
      Niveau de finalisation : moyen\\
      \begin{verbatim}
        #include "imagetoolsmorpho.H"
      \end{verbatim}
    @author \URL[Bastien Peteuil]{mailto:bpeteuil@insa-rouen.fr}
    @author \URL[Carole Le Bris]{mailto:clebris@irisa.fr}
    @author \URL[Franck Galpin]{mailto:fgalpinl@irisa.fr}
*/
//@{
/** Classe ImageToolsMorpho */
template <class T>
class ImageToolsMorpho
{
public:

  /** @doc Methode d'erosion d'une image binaire
      @param image une instance de la classe image
      @param w_size la taille de l'element structurant (suppose carre)
      @param elt Une matrice de booleen representant l'lment structurant (l'application se fait au centre)
      @return l'image resultat
   */
  static Image<T> Erosion(const Image<T> &image,const int w_size,bool *elt) ;

  /** @doc Methode de dilatation d'une image binaire
      @param image une instance de la classe image
      @param w_size la taille de l'element structurant (suppose carre)
      @param elt Une matrice de booleen representant l'lment structurant (l'application se fait au centre)
      @return l'image resultat
   */
  static Image<T> Dilatation(const Image<T> &image,const int w_size,bool *elt);

  /** @doc permet de faire la erosion d une image de type quelconque en morphologie mathematique binaire
      @param image image a traiter
      @param objeter image de l element structurant 0 vide, 1 point appartenant a element, 2 son centre
      @param res image resultat de la erosion de image par objeter
      @return int 1 pas de probleme, 0 objeter trop grand pour l image
  */
  static int Erosion2(const Image<T> &image, const Image<T> &objet, Image<T> &res);

  /** @doc permet de faire la dilatation d une image de type quelconque en morphologie mathematique binaire
      @param image image a traiter
      @param objeter image de l element structurant 0 vide, 1 point appartenant a element, 2 son centre
      @param res image resultat de la dilatation de image par objeter
      @return int 1 pas de probleme, 0 objeter trop grand pour l image
  */
  static int Dilatation2(const Image<T> &image, const Image<T> &objet, Image<T> &res);
};
//@}



template <class T>
Image<T> ImageToolsMorpho<T>::Dilatation(const Image<T> &image1,const int size,bool *elt)
{
  int w_size=(int)(size/2.);
  int nbl=image1.NbRow();
  int nbc=image1.NbCol();
  Image<T> image(nbl,nbc);;
  bool test;
  int i,j,u,v,deb_i=0,deb_j=0,fin_i=0,fin_j=0;

  for(i=0;i<nbc;i++)
    {
      for(j=0;j<nbl;j++)
	{
	  test=false;
	  if((i>=w_size)&&(i<nbc-1-w_size)&&(j>=w_size)&&(j<nbl-1-w_size))
	  {
	    for(u=i-w_size;u<=i+w_size;u++)
	    {
	      for(v=j-w_size;v<=j+w_size;v++)
	      {
		if(test)
		  break;

		if(elt[(w_size+v-j)*size+(w_size+u-i)])
		{
		  if(image1(v,u)!=0)
		     test=true;
		}
	      }
	    }

	      image(j,i)=test*255;

	  }
	  else
	  {

	    /* haut gauche */
	    if((i<w_size)&&(j<w_size))
	    {
	      deb_i=0;
	      deb_j=0;
	      fin_i=w_size;
	      fin_j=w_size;
	    }
	    /* gauche */
	    if((i<w_size)&&(j>=w_size)&&(j<nbl-1-w_size))
	    {
	      deb_i=0;
	      deb_j=j-w_size;
	      fin_i=w_size;
	      fin_j=j+w_size;
	    }

	    /* bas gauche */
	    if((i<w_size)&&(j>=nbl-1-w_size))
	    {
	      deb_i=0;
	      deb_j=j-w_size;
	      fin_i=w_size;
	      fin_j=nbl-1;
	    }

	    /* haut droit */
	    if((i>=nbc-1-w_size)&&(j<w_size))
	    {
	      deb_i=i-w_size;
	      deb_j=0;
	      fin_i=nbc-1;
	      fin_j=j+w_size;
	    }

	    /* droit */
	    if((i>=nbc-1-w_size)&&(j>=w_size)&&(j<nbl-1-w_size))
	    {
	      deb_i=i-w_size;
	      deb_j=j-w_size;
	      fin_i=nbc-1;
	      fin_j=j+w_size;
	    }


	    /* bas droit */
	    if((i>=nbc-1-w_size)&&(j>=nbl-1-w_size))
	    {
	      deb_i=i-w_size;
	      deb_j=j-w_size;
	      fin_i=nbc-1;
	      fin_j=nbl-1;
	    }

	    /* haut */

	    if((i<nbc-1-w_size)&&(j<w_size)&&(i>=w_size))
	    {
	      deb_i=i-w_size;
	      deb_j=0;
	      fin_i=i+w_size;
	      fin_j=w_size;
	    }

	    /* bas */

	    if((i<nbc-1-w_size)&&(j>=nbl-1-w_size)&&(i>=w_size))
	    {
	      deb_i=i-w_size;
	      deb_j=j-w_size;
	      fin_i=i+w_size;
	      fin_j=nbl-1;
	    }

	    for(u=deb_i;u<=fin_i;u++)
	    {
	      for(v=deb_j;v<=fin_j;v++)
	      {
		if(test)
		  break;

		if(elt[(w_size+v-j)*size+(w_size+u-i)])
		  if(image1(v,u)!=0)
		    test=true;
	      }
	    }
	    image(j,i)=test*255;


	  }/* fin du if hors centre */

	}
    }
  return image;
}

template <class T>
Image<T> ImageToolsMorpho<T>::Erosion(const Image<T> &image1,const int size,bool *elt)
{
  int w_size=(int)(size/2.);
  int nbl=image1.NbRow();
  int nbc=image1.NbCol();
  Image<T> image(nbl,nbc);;
  bool test;
  int i,j,u,v,deb_i=0,deb_j=0,fin_i=0,fin_j=0;

  for(i=0;i<nbc;i++)
    {
      for(j=0;j<nbl;j++)
	{

	  test=false;
	  if((i>=w_size)&&(i<nbc-1-w_size)&&(j>=w_size)&&(j<nbl-1-w_size))
	  {
	    for(u=i-w_size;u<=i+w_size;u++)
	    {
	      for(v=j-w_size;v<=j+w_size;v++)
	      {
		if(test)
		  break;

		if(elt[(w_size+v-j)*size+(w_size+u-i)])
		{
		  if(image1(v,u)==0)
		     test=true;
		}
	      }
	    }

	      image(j,i)=(!test)*255;

	  }
	  else
	  {

	    /* haut gauche */
	    if((i<w_size)&&(j<w_size))
	    {
	      deb_i=0;
	      deb_j=0;
	      fin_i=w_size;
	      fin_j=w_size;
	    }
	    /* gauche */
	    if((i<w_size)&&(j>=w_size)&&(j<nbl-1-w_size))
	    {
	      deb_i=0;
	      deb_j=j-w_size;
	      fin_i=w_size;
	      fin_j=j+w_size;
	    }

	    /* bas gauche */
	    if((i<w_size)&&(j>=nbl-1-w_size))
	    {
	      deb_i=0;
	      deb_j=j-w_size;
	      fin_i=w_size;
	      fin_j=nbl-1;
	    }

	    /* haut droit */
	    if((i>=nbc-1-w_size)&&(j<w_size))
	    {
	      deb_i=i-w_size;
	      deb_j=0;
	      fin_i=nbc-1;
	      fin_j=j+w_size;
	    }

	    /* droit */
	    if((i>=nbc-1-w_size)&&(j>=w_size)&&(j<nbl-1-w_size))
	    {
	      deb_i=i-w_size;
	      deb_j=j-w_size;
	      fin_i=nbc-1;
	      fin_j=j+w_size;
	    }


	    /* bas droit */
	    if((i>=nbc-1-w_size)&&(j>=nbl-1-w_size))
	    {
	      deb_i=i-w_size;
	      deb_j=j-w_size;
	      fin_i=nbc-1;
	      fin_j=nbl-1;
	    }

	    /* haut */

	    if((i<nbc-1-w_size)&&(j<w_size)&&(i>=w_size))
	    {
	      deb_i=i-w_size;
	      deb_j=0;
	      fin_i=i+w_size;
	      fin_j=w_size;
	    }

	    /* bas */

	    if((i<nbc-1-w_size)&&(j>=nbl-1-w_size)&&(i>=w_size))
	    {
	      deb_i=i-w_size;
	      deb_j=j-w_size;
	      fin_i=i+w_size;
	      fin_j=nbl-1;
	    }

	    for(u=deb_i;u<=fin_i;u++)
	    {
	      for(v=deb_j;v<=fin_j;v++)
	      {
		if(test)
		  break;

		if(elt[(w_size+v-j)*size+(w_size+u-i)])
		  if(image1(v,u)==0)
		    test=true;
	      }
	    }
	    image(j,i)=(!test)*255;


	  }/* fin du if hors centre */

	}
    }
  return image;
}


template <class T>
int ImageToolsMorpho<T>::Erosion2(const Image<T> &image, const Image<T> &objet, Image<T> &res) {
  short Ci,Cj; //position du centre de l objet repere par 2
  int Inbl, Inbc, Onbl, Onbc;
  short i,j,k,l;
  bool fait;
  T maxval=0;  // pour warning
  Image<T> objeter;
  objeter=objet;

  //initialisation du resultat
  res=image;

  //initialisation du nombre de lignes et de colonnes
  Inbl=image.NbRow();
  Inbc=image.NbCol();
  Onbl=objeter.NbRow();
  Onbc=objeter.NbCol();

  if(Onbl<=Inbl && Onbc<=Inbc){
    //recuperation de la position du centre
    i=-1;
    j=Onbc;
    while(i<Onbl && j==Onbc)
    {
      j=0;
      i++;
      while(j<Onbc && objeter(i,j)!=2) j++;
    }

    Ci=i;
    Cj=j;

    //mise a 1 du centre pour operer l erosion
    objeter(Ci,Cj)=1;

    //erosion de limage
    for(i=0;i<Inbl;i++)
      for(j=0;j<Inbc;j++)
      {
	fait=false;
	for(k=(-Ci);k<Onbl-Ci;k++)
	  for(l=(-Cj);l<Onbc-Cj;l++)
	    if(i+k>=0 && j+l>=0 && j+l<Inbc && i+k<Inbl)//le point de l objet appartient a l image
	    {if(objeter(k+Ci,l+Cj)==1 && fait)
	      if(maxval>image(i+k,j+l)) maxval=image(i+k,j+l);
	    if(objeter(k+Ci,l+Cj)==1 && !fait)
	    {
	      fait=true;
	      maxval=image(i+k,j+l);}
	    }
	res(i,j)=maxval;
      }

    return(1);
  }
  else return(0);
}

template <class T>
int ImageToolsMorpho<T>::Dilatation2(const Image<T> &image, const Image<T> &objet, Image<T> &res) {
  short Ci,Cj,nCi,nCj; //position du centre de l objet repere par 2
  int Inbl, Inbc, Onbl, Onbc;
  short i,j,k,l;
  T maxval=0; // pour warning
  Image<T> objeter(objet.NbRow(),objet.NbCol()); //image de l objet translate
  bool fait;

  //initialisation du resultat
  res=image;

  //initialisation du nombre de lignes et de colonnes
  Inbl=image.NbRow();
  Inbc=image.NbCol();
  Onbl=objet.NbRow();
  Onbc=objet.NbCol();

  if(Onbl<=Inbl && Onbc<=Inbc)
  {//recuperation de la position du centre
    i=-1;
    j=Onbc;
    while(i<Onbl && j==Onbc)
    {
      j=0;
      i++;
      while(j<Onbc && objet(i,j)!=2) j++;
    }

    Ci=i;
    Cj=j;

    //la dilatation utilisa le translate
    //creation du translate
    for(i=0;i<Onbl;i++)
      for(j=0;j<Onbc;j++)
	objeter(i,j)=0;

    nCj=Onbc-1-Cj;
    nCi=Onbl-1-Ci;
    objeter(nCi,nCj)=2;

    for(i=0;i<Onbl;i++)
      for(j=0; j<Onbc;j++)
	if(objet(i,j)==1)
	  objeter(-(i-Ci)+nCi,-(j-Cj)+nCj)=1;

    //mise a 1 du centre pour operer la dilatation
    objeter(nCi,nCj)=1;

    for(i=0;i<Inbl;i++)
      for(j=0;j<Inbc;j++)
      {
	fait=false;
	for(k=(-nCi);k<Onbl-nCi;k++)
	  for(l=(-nCj);l<Onbc-nCj;l++)
	    if(i+k>=0 && j+l>=0 && j+l<Inbc && i+k<Inbl)//le point de l objet appartient a l image
	    {
	      if(objeter(k+nCi,l+nCj)==1 && fait )
		if(maxval<image(i+k,j+l)) maxval=image(i+k,j+l);
	      if(objeter(k+nCi,l+nCj)==1 && !fait)
	      {
		fait=true;
		maxval=image(i+k,j+l);}
	    }
	res(i,j)=maxval;
      }

    return(1);
  } else return(0);
}

#endif
